/*  Portbelegungen und Konfiguration:
    =================================
 */

#include "StdTypes.h"
#include "startup.h"
#include "config.h"
#include "systick.h"
#include "serial.h"


void Do_Config (void)
{ long L;

  RCC->CIR      = 0;           /* alle RCC-Interrupts ausschalten */
  RCC->APB1RSTR = 0;           /* evtl. Reset's beseitigen */
  RCC->APB2RSTR = 0;
  FLASH->ACR  = (1<<4);        /* Flash-Prefetch-Buffer einschalten */
  RCC->CFGR   = (5<<24);       /* MCO=HSI, sonst nix.. erstmal */
  RCC->AHBENR = 0xFFFF;        /*  */
  RCC->APB2ENR = 0xFFFFFFFF;
  RCC->APB2ENR = 0xFFFF;

  GPIOG->CRL = wf_GPIOG_CRL;
  GPIOG->CRH = wf_GPIOG_CRH;

  GPIOG->ODR &= ~(1<<6);
  L = 10000; while (--L);


/* zuerst Takterzeugung aufsetzen.
     nach einem Hardware-Reset sind folgende Bits gesetzt:
     in RCC_CR     = HSIRDY, HSION, HSITRIM = 8
     in RCC_AHBENR = FLITFEN, SRAMEN
     in RCC_CSR    = PORRSTF, PINRSTF
     Vorsicht: nach irgendwelchen Software-Resets mu das nicht so sein!
 */
  RCC->CR    =  (8<<2)|1;          /* Umschalten auf HSI = interne 8 MHz */
  while ((RCC->CR & 2)==0);        /* warten auf HSIRDY (wird von Hardware gesetzt) */


  /* so, jetzt ist RCC im Urzustand wie nach einem Hardware-Reset */

  RCC->CR |= (1<<16);       /* externen 6 MHz Oszillator HSE einschalten */
  L = 5000;
  while (L)
  { --L; if (RCC->CR & (1<<17)) goto _hse_rdy; }         /* wenn ext. Oszi ready */
  _hse_rdy:


  if (L)                          /* wenn wir nicht vergeblich auf HSE gewartet haben.. */
  {  FLASH->ACR  = (1<<4)|2;      /* Prefetch-Buffer + 2 Waits bei 72 MHz  */
     RCC->CFGR   = (7<<24)|       /* MCO=1/2PLL */
                   (0<<22)|       /* UsbClk=PLL/1.5 */
                   (10<<18)|      /* PLL = 12*HSE (6*12=72) */
                   (1<<16)|       /* PllSource=HSE */
                   (3<<14)|       /* AdcTakt=PCLK2/8 */
                   (4<<11)|       /* APB2=Hclk/2=36MHz */
                   (4<<8);        /* APB1=Hclk/2=36MHz */

     RCC->CR    |= (1<<24);       /* PLL einschalten */

     while ((RCC->CR & (1<<25))==0);           /* warten auf PLLRDY */
     RCC->CFGR  = (RCC->CFGR & 0xFFFFFFFC)|2;  /* PLL als Taktquelle whlen */
     do { L = RCC->CFGR & (3<<2);
        } while (L!= (2<<2));               /* warten bis PLL=Taktquelle */
  }

  RCC->AHBENR = wf_AHBENR;    /* AHB enables  */
  RCC->APB1ENR = wf_APB1ENR;  /* APB1 enables */
  RCC->APB2ENR = wf_APB2ENR;  /* APB2 enables */


 /* Backup-Domain enables:
    sind nach Reset erstmal schreibgeschtzt. Entsperren mit PWR->CR (1<<8) auf hi  */
  if ((RCC->BDCR & 2)==0)          /* wenn 32kHz Oszi nicht ready ist */
  { PWR->CR |= (1<<8);
    RCC->BDCR = (1<<16)|(1<<15)|1; /* RTC Reset, enable, 32kHz on */
    RCC->BDCR = (1<<15)|(1<<8)|1;  /* RTC enable, LSE als Quelle, 32kHz on */
  }


/*  Pin-Funktionen aufsetzen */
 GPIOA->CRL = wf_GPIOA_CRL;
 GPIOA->CRH = wf_GPIOA_CRH;

 GPIOB->CRL = wf_GPIOB_CRL;
 GPIOB->CRH = wf_GPIOB_CRH;

 GPIOC->CRL = wf_GPIOC_CRL;
 GPIOC->CRH = wf_GPIOC_CRH;

 GPIOD->CRL = wf_GPIOD_CRL;
 GPIOD->CRH = wf_GPIOD_CRH;

 GPIOE->CRL = wf_GPIOE_CRL;
 GPIOE->CRH = wf_GPIOE_CRH;

 GPIOF->CRL = wf_GPIOF_CRL;
 GPIOF->CRH = wf_GPIOF_CRH;

 GPIOG->CRL = wf_GPIOG_CRL;
 GPIOG->CRH = wf_GPIOG_CRH;

/* Pin-Remap */
 AFIO->MAPR = wf_AFIO_MAPR;


/************* Bus aufsetzen **************************************/

/* Der externe RAM ist 16 Bit breit (2xK6X4008C1F-VF55)
   Da der Controller zu bld ist, um 2 separate 8 Bit Chips zu verwalten,
   erledigt das eine externe Hardware:
   /CE8-15:= NE1 or NBL1;
   /CE0-7 := NE1 or NBL0;
   Zugriffszeit ist 55 ns

   Das CPLD wird wie ein langsamer 16 Bit RAM behandelt.
   Zugriffszeit mind. 150 ns
 */
  FSMC_BTR1 = (4<<8);
  FSMC_BCR1 = (1<<12) | (1<<4) | 1;

  FSMC_BTR2 = (11<<8);
  FSMC_BCR2 = (1<<12) | (1<<4) | 1;
}



dword  ExtRamSize;

/* ein billiger RAM Test */
void ExtRamTest(void)
{ dword* PDW;
  dword* PDW0;
  dword* PDEND;

  PDEND = (dword*) ExtRamBegin + 0x400000;

  PDW0 = PDW = (dword*) ExtRamBegin;
  *PDW0 = 0x4711AFFE;
  ++PDW;

  while ((PDW < PDEND) && (*PDW0 == 0x4711AFFE))
  { *PDW = 0xFFFEFFFE;
    ++PDW;
  }

  *PDW0 = 0xFFFEFFFE;
  while (PDW0 < PDW)
  { if (*PDW0 == 0xFFFEFFFE) *PDW0 = *PDW0 + 0x10002;
    ++PDW0;
  }

  PDW0 = (dword*)ExtRamBegin;
  while ((PDW0 < PDW) && (*PDW0==0)) ++PDW0;
  ExtRamSize = (dword)PDW0 - (dword)ExtRamBegin;
}
